#ifndef DISSECT_C
#define DISSECT_C

#include "dissect.h"

//ethernet dissector table//////////////////////////////////////////////////////
struct dissector eth_table[] = 
{
  {0x0800,"(IP)",(void*)parse_ip_pkt},
  {0x0806,"(ARP/RARP)",(void*)parse_arp},
  {0x814C,"(SNMP)",(void*)0},
  {0x8037,"(IPX)",(void*)0},
  {0x86DD,"(IPv6)",(void*)0},
  {0x0000,"STOP",(void*)0}
};

//dissect ethernet frame/////////////////////////////////////////////////////
void parse_eth_frame(unsigned char* data, unsigned int len)
{
  unsigned int i;

  //if verbosity is not none or low, print tiimestamp
  if(verbosity == 3 || verbosity ==2)
  {  
    vga_dec_output(hours);
    vga_write_output(':');
    vga_dec_output(minutes);
    vga_write_output(':');
    vga_dec_output(seconds);
    vga_write_output(' ');
  }
  
  //print if verbosity high
  if(verbosity ==3)
  {
    vga_output("Frame Received, length ");
    vga_dec_output(len);
    vga_output(" bytes, status ");
    vga_hex_output(ior(6),1);
    vga_return_output();

    //print src mac
    vga_hex_output(data[6],1);
    vga_hex_output(data[7],0);
    vga_hex_output(data[8],0);
    vga_hex_output(data[9],0);
    vga_hex_output(data[10],0);
    vga_hex_output(data[11],0);
    
    vga_write_output(ARROWRT);
    
    //print dest mac
    vga_hex_output(data[0],1);
    vga_hex_output(data[1],0);
    vga_hex_output(data[2],0);
    vga_hex_output(data[3],0);
    vga_hex_output(data[4],0);
    vga_hex_output(data[5],0);
    
    vga_output("  ");
  } 
     
  //extract type/length field
  long unsigned int type;
  type = data[12] << 8;
  type = ((type)|(data[13]));
  
  //only print this if verbosity is high
  if(verbosity == 3)
  {
    //print type/802.3 length field
    if(type < 0x600)
    {
      vga_output("\n802.3 Length: ");
      vga_dec_output(type);
    }
    else
    {
      vga_output("\nFrame Type: ");
      vga_hex_output(data[12],1);
      vga_hex_output(data[13],0);
    }
  }

  //log everything if verbosity == 3, else only do timestamp
  log_dec_data(hours);
  log_data(":");
  log_dec_data(minutes);
  log_data(":");
  log_dec_data(seconds);
  log_data(" ");
  if(verbosity==3)
  {
    log_data("Frame Received, length ");
    log_dec_data(len);
    log_data(" bytes, status ");
    log_hex_data(ior(6),1);
    log_data("\n");

    //print src mac
    log_hex_data(data[6],1);
    log_hex_data(data[7],0);
    log_hex_data(data[8],0);
    log_hex_data(data[9],0);
    log_hex_data(data[10],0);
    log_hex_data(data[11],0);
    
    log_char(ARROWRT);
    
    //print dest mac
    log_hex_data(data[0],1);
    log_hex_data(data[1],0);
    log_hex_data(data[2],0);
    log_hex_data(data[3],0);
    log_hex_data(data[4],0);
    log_hex_data(data[5],0);
    
    log_data("  ");
    
    //print type/802.3 length field
    if(type < 0x600)
    {
      log_data("\n802.3 Length: ");
      log_hex_data(type,1);
    }
    else
    {
      log_data("\nFrame Type: ");
      log_hex_data(data[12],1);
      log_hex_data(data[13],0);
    }
  }

  //pass to next dissector
  for(i=0;eth_table[i].typeid!=0;i++)
  {
    if(type==eth_table[i].typeid) 
    {
      if (verbosity == 3)
      {
        vga_write_output(SPACE);
        vga_output(eth_table[i].name);
        vga_return_output();
        
        log_data(" ");
        log_data(eth_table[i].name);
        log_data("\n");
        
      }
      if(eth_table[i].dissectFcn != 0)
        (eth_table[i].dissectFcn)(data+14,len-14);
      break;
    }
  }
  if(eth_table[i].typeid == 0 && verbosity!=0)
  {
    vga_output(" (Unknown Protocol)");
    log_data(" (Unknown Protocol) ");
  }

  //if there is any output, print two line returns.
  if(verbosity != 0 && verbosity != 1)
  {
    vga_return_output();
    vga_return_output();
    log_data("\n\n");
  }
}

//IP dissector table////////////////////////////////////////////////////////////
struct dissector ip_table[] = 
{
  {6,"(TCP)",(void*)parse_tcp_dgram},
  {17,"(UDP)",(void*)parse_udp_dgram},
  {1,"(ICMP)",(void*)parse_icmp},
  {9,"(IGRP)",(void*)0},
  {88,"(EIGRP)",(void*)0},
  {89,"(OSPF)",(void*)0},
  {0x0000,"STOP",(void*)0}
};

//parse IP packet///////////////////////////////////////////////////////////////
void parse_ip_pkt(unsigned char* data, unsigned int len)
{ 
  unsigned short int i;
  
  //get header length 
  unsigned short int length = (data[0] & 0x0F)<<2;
  
  //only do this if verbosity = high
  if(verbosity ==3)
    vga_output("  Internet Protocol version 4\n  ");
  
  //only do this if verbosity is high/medium
  if(verbosity ==3 || verbosity==2)
  {
    //print src/dest ip
    vga_dec_output(data[12]);
    vga_write_output('.');
    vga_dec_output(data[13]);
    vga_write_output('.');
    vga_dec_output(data[14]);
    vga_write_output('.');
    vga_dec_output(data[15]);
    vga_write_output(ARROWRT);
    vga_dec_output(data[16]);
    vga_write_output('.');
    vga_dec_output(data[17]);
    vga_write_output('.');
    vga_dec_output(data[18]);
    vga_write_output('.');
    vga_dec_output(data[19]);
    vga_write_output(SPACE);
  }
  
  //only do this if verbosity is high
  if(verbosity ==3)
  {
    //print Differentiated Services Codepoint
    vga_output("\n  DSCP value: ");
    vga_hex_output(data[1],1);
    
    //print Time To Live
    vga_output("  TTL: ");
    vga_dec_output(data[8]);
    
    //print checksum
    vga_output("\n  Checksum: ");
    vga_hex_output(data[10],1);
    vga_hex_output(data[11],0);
    
    //print header length in bytes
    vga_output("  Opt: ");
    if (length != 20)
    {
      vga_hex_output(data[20],1);
      for(i=21;i<length;i++)
      {
        vga_hex_output(data[i],0);
      }
    }
    else
    {
      vga_output("None"); 
    }
    
    //identify upper-layer protocol
    vga_output("\n  Protocol: ");
    vga_hex_output(data[9],1);
  }
  
  //pass off to next dissector
  unsigned long int type = data[9];
  for(i=0;ip_table[i].typeid!=0;i++)
  {
    if(type==ip_table[i].typeid) 
    {
      if(verbosity == 3)
      {
        vga_write_output(SPACE);
        vga_output(ip_table[i].name);
      }
      if(ip_table[i].dissectFcn!=0)
        (ip_table[i].dissectFcn)(data+length,len-length);
      break;
    }
  }
  if(ip_table[i].typeid == 0 && verbosity !=0)
  {
    vga_output(" (Unknown Protocol)"); 
  }
}

//dissect arp message///////////////////////////////////////////////////////////
//TODO: these are the common ones, do the other 6 messages.
//alternatively, they all use the same format, just make the below stuff general.
void parse_arp(unsigned char* data, unsigned int len)
{
  unsigned short int i;
  
  //print this unless verbosity = none
  if(verbosity !=0)
  {
    if(verbosity ==3)
    vga_output("  Address Resolution Protocol\n  "); 
    //get ARP type
    switch(((data[6])<<8)||data[7])
    {
      //ARP request - Who has xxx.xxx.xxx.xxx?  Tell yyy.yyy.yyy.yyy.
      case 1:
        vga_output("ARP REQUEST: Who has ");
        for(i=24;i<27;i++)
        {
          vga_dec_output(data[i]);
          vga_write_output('.');
        }
        vga_dec_output(data[i]);
        vga_output("?  Tell ");
        for(i=14;i<18;i++)
        {
          vga_dec_output(data[i]);
          vga_write_output('.');
        }
        vga_return_output();
        break;
    
      //ARP reply - Hello, yyy.yyy.yyy.yyy.  I have xxx.xxx.xxx.xxx.
      case 2:
        vga_output("ARP REPLY: Hello ");
        for(i=24;i<28;i++)
        {
          vga_dec_output(data[i]);
          vga_write_output('.');
        }
        vga_output("  I have ");
        for(i=14;i<18;i++)
        {
          vga_dec_output(data[i]);
          vga_write_output('.');
        }
        vga_return_output();
        break;
    
      //RARP request - Who has 0xXXXXXXXXXX? Tell 0xYYYYYYYYYYYY.
      case 3:
        vga_output("RARP REQUEST: Who has ");
        vga_hex_output(data[18],1);
        for(i=19;i<24;i++)
        {
          vga_hex_output(data[i],0);
        }
        vga_output("?  Tell ");
        vga_hex_output(data[8],1);
        for(i=9;i<13;i++)
        {
          vga_hex_output(data[i],0);
        }
        vga_return_output();
        break;
    
      //RARP reply - Hello, 0xYYYYYYYYYYYY.  I have 0xXXXXXXXXXXXX.
      case 4:
        vga_output("RARP REPLY: Hello ");
        vga_hex_output(data[18],1);
        for(i=19;i<24;i++)
        {
          vga_hex_output(data[i],0);
        }
        vga_output(".  I have ");
        vga_hex_output(data[8],1);
        for(i=9;i<13;i++)
        {
          vga_hex_output(data[i],0);
        }
        vga_return_output();
        break;
        
      //default - inARP and DRARP are unimplemented, just return
      default:
        break;
    }
  }
}

//dissect ICMP messages/////////////////////////////////////////////////////////
//TODO: these are the common ones, do the other 30 or so ICMP messages
void parse_icmp(unsigned char* data, unsigned int len)
{
  if(verbosity != 0)
  {
    unsigned short int i=0;
    vga_output("\n    Internet Control Message Protocol\n");
    vga_output("    Type: ");
    switch(data[0])
    {
      case 0:
        vga_output("Echo reply  ");
        vga_output("Checksum: ");
        vga_hex_output(data[2],1);
        vga_hex_output(data[3],0);
        vga_output("\n    Identifier: ");
        vga_hex_output(data[4],1);
        vga_hex_output(data[5],0);
        vga_output("  Seq. #: ");
        vga_hex_output(data[6],1);
        vga_hex_output(data[7],0);
        vga_output("\n    Data: ");
        vga_write_output(data[8]);
        for(i=9;i<len;i++)
          vga_write_output(data[i]);
        break;
      case 3:
        vga_output("Destination unreachable ");
        vga_output("Code: ");
        vga_hex_output(data[1],1);
        vga_output("\n    Checksum: ");
        vga_hex_output(data[2],1);
        vga_hex_output(data[3],0);
        vga_output("  Next-hop MTU: ");
        vga_dec_output(((data[6])<<8)|data[7]);
        break;
      case 5:
        vga_output("Redirect ");
        vga_output("Code: ");
        vga_hex_output(data[1],1);
        vga_output("\n    Checksum: ");
        vga_hex_output(data[2],1);
        vga_hex_output(data[3],0);
        vga_output("  Redirect to: ");
        for(i=4;i<8;i++)
        {
          vga_dec_output(data[i]);
          if(i==7)
            break;
          vga_write_output('.');
        }
        break;
      case 8:
        vga_output("Echo request  ");
        vga_output("Checksum: ");
        vga_hex_output(data[2],1);
        vga_hex_output(data[3],0);
        vga_output("\n    Identifier: ");
        vga_hex_output(data[4],1);
        vga_hex_output(data[5],0);
        vga_output("  Seq. #: ");
        vga_hex_output(data[6],1);
        vga_hex_output(data[7],0);
        vga_output("\n    Data: ");
        vga_write_output(data[8]);
        for(i=9;i<len;i++)
          vga_write_output(data[i]);
        break;
      case 11:
        vga_output("Time exceeded ");
        vga_output("Code: ");
        vga_hex_output(data[1],1);
        vga_output("\n    Checksum: ");
        vga_hex_output(data[2],1);
        vga_hex_output(data[3],0);
        break;
      default:
        vga_hex_output(data[1],1);
        vga_output(" (Unknown) ");
        vga_output("Code: ");
        vga_hex_output(data[1],1);
        vga_output("\n    Checksum: ");
        vga_hex_output(data[2],1);
        vga_hex_output(data[3],0);
        break;
    }
  }
}

//TCP dissector table///////////////////////////////////////////////////////////
//TODO:  This list is FFFAAAAAAARRRR from complete, I only listed the ports that
//might be useful for surveillance/infiltration and only parse the ones useful
//for eavesdropping/monitoring.  It would be really time-consuming to do all these.
//Maybe I should make this an open-source project and get some other nerds to help.
struct dissector tcp_table[] =
{
  {7,"(echo)",(void*)0},
  {13,"(daytime)",(void*)0},
  {17,"(qotd)",(void*)0},
  {20,"(FTP data)",(void*)parse_ftp_data},
  {21,"(FTP control)",(void*)parse_ftp_control},
  {22,"(SSH)",(void*)0},
  {23,"(Telnet)",(void*)parse_telnet},
  {25,"(SMTP)",(void*)parse_smtp},
  {49,"(TACACS)",(void*)0},
  {53,"(DNS)",(void*)parse_dns},
  {63,"(whois)",(void*)0},
  {65,"(TACACS-DS)",(void*)0},
  {66,"(Oracle SQL)",(void*)0},
  {67,"(BOOTP/DHCP server)",(void*)parse_bootp_dhcp_server},
  {68,"(BOOTP/DHCP client)",(void*)parse_bootp_dhcp_client},
  {69,"(TFTP)",(void*)parse_tftp},
  {79,"(Finger)",(void*)0},
  {80,"(HTTP)",(void*)parse_http},
  {81,"(HTTPS)",(void*)0},
  {88,"(Kerberos)",(void*)0},
  {110,"(POP3)",(void*)parse_pop3},
  {111,"(SunRPC)",(void*)0},
  {119,"(NNTP)",(void*)0},
  {137,"(Netbios-NS)",(void*)0},
  {138,"(Netbios-DGM)",(void*)0},
  {139,"(Netbios-SSN)",(void*)0},
  {143,"(IMAP)",(void*)parse_imap},
  {194,"(IRC)",(void*)0},
  {389,"(LDAP)",(void*)0},
  {465,"(SMTPS)",(void*)0},
  {636,"(LDAPS)",(void*)0},
  {179,"(BGP)",(void*)0},
  {5190,"(AIM)",(void*)parse_aim},
  {995,"(POP3S)",(void*)0},
  {5800,"(VNC)",(void*)0},
  {0,"STOP",(void*)0}
};


//dissect tcp datagram//////////////////////////////////////////////////////////
void parse_tcp_dgram(unsigned char* data, unsigned int len)
{
  //identify upper-layer protocol - check both src and dest port and use the well-known one
  short int i=0, dissector = -1;
  unsigned long int spt = (((data[0])<<8)|data[1]);
  unsigned long int dpt = (((data[2])<<8)|data[3]);
  int offset = (data[12] & 0xF0) >> 2;
  for(i=0;tcp_table[i].typeid!=0;i++)
  {
    if(spt==tcp_table[i].typeid) 
    {
      dissector = i;
      break;
    }
    if(dpt==tcp_table[i].typeid) 
    {
      dissector = i;
      break;
    }
  }
  
  //if verbosity == high
  if(verbosity ==3)
    vga_output("\n   Transmission Control Protocol\n");
  
  //print that it's a tcp port if medium verbosity
  if(verbosity ==2)
    vga_output("TCP port ");
  
  //if verbosity == high or medium
  if(verbosity ==3 || verbosity ==2)
  {
    if(verbosity ==3)
      vga_output("    Port ");
    vga_dec_output(spt);
    vga_write_output(ARROWRT);
    if(verbosity ==3)
      vga_output("Port ");
    vga_dec_output(dpt);
  }
  
  if(verbosity == 2)
  {
    if(dissector != -1)
    {
      vga_write_output(SPACE);
      vga_output(tcp_table[dissector].name);
    }
    else
    {
      vga_output(" (Unknown Protocol)"); 
    }
  }
  
  //if verbosity is high
  if(verbosity ==3)
  {
    if(dissector != -1)
    {
      vga_write_output(SPACE);
      vga_output(tcp_table[dissector].name);
    }
    vga_output("\n    Seq#: ");
    vga_hex_output(data[4],1);
    vga_hex_output(data[5],0);
    vga_hex_output(data[6],0);
    vga_hex_output(data[7],0);
    vga_output("  Ack#: ");
    vga_hex_output(data[8],1);
    vga_hex_output(data[9],0);
    vga_hex_output(data[10],0);
    vga_hex_output(data[11],0);
    vga_output("\n    Checksum: ");
    vga_hex_output(data[16],1);
    vga_hex_output(data[17],0);
    vga_output("  Urgent: ");
    vga_hex_output(data[18],1);
    vga_hex_output(data[19],0); 
    vga_output("\n    Window: ");
    vga_hex_output(data[14],1);
    vga_hex_output(data[15],0);
    vga_output("    Offset: ");
    vga_hex_output(offset,1);
    vga_output("\n    Flags: ");
    if(data[13] & 0x20)
    {
      vga_output("URG ");
    }
    if(data[13] & 0x10)
    {
      vga_output("ACK "); 
    }
    if(data[13] & 0x08)
    {
      vga_output("PSH "); 
    }
    if(data[13] & 0x04)
    {
      vga_output("RST "); 
    }
    if(data[13] & 0x02)
    {
      vga_output("SYN "); 
    }
    if(data[13] & 0x01)
    {
      vga_output("FIN "); 
    }
  }
  
  //call upper-layer dissector
  if(dissector != -1)
  {
    if(tcp_table[dissector].dissectFcn != 0)
      (tcp_table[i].dissectFcn)(data+offset,len-offset);
  }
}
//udp dissector table///////////////////////////////////////////////////////////
struct dissector udp_table[] = 
{
  {7,"(echo)",(void*)0},
  {13,"(daytime)",(void*)0},
  {17,"(qotd)",(void*)0},
  {20,"(FTP data)",(void*)parse_ftp_data},
  {21,"(FTP control)",(void*)parse_ftp_control},
  {22,"(SSH)",(void*)0},
  {23,"(Telnet)",(void*)parse_telnet},
  {25,"(SMTP)",(void*)parse_smtp},
  {49,"(TACACS)",(void*)0},
  {53,"(DNS)",(void*)parse_dns},
  {63,"(whois)",(void*)0},
  {65,"(TACACS-DS)",(void*)0},
  {66,"(Oracle SQL)",(void*)0},
  {67,"(BOOTP/DHCP server)",(void*)parse_bootp_dhcp_server},
  {68,"(BOOTP/DHCP client)",(void*)parse_bootp_dhcp_client},
  {69,"(TFTP)",(void*)parse_tftp},
  {111,"(SunRPC)",(void*)0},
  {123,"(NTP)",(void*)parse_ntp},
  {137,"(Netbios-NS)",(void*)0},
  {138,"(Netbios-DGM)",(void*)0},
  {139,"(Netbios-SSN)",(void*)0},
  {143,"(IMAP)",(void*)parse_imap},
  {389,"(LDAP)",(void*)0},
  {1900,"(SSDP)",(void*)0},
  {5004,"(RTP)",(void*)0},
  {5005,"(RTP)",(void*)0},
  {5060,"(SIP)",(void*)0},
  {0,"STOP",(void*)0}
};


//dissect udp datagram//////////////////////////////////////////////////////////
void parse_udp_dgram(unsigned char* data, unsigned int len)
{
  //identify upper-layer protocol - check both src and dest port and use the well-known one
  short int i=0, dissector = -1;
  int length = (((data[4])<<8)|data[5]);
  unsigned long int spt = (((data[0])<<8)|data[1]);
  unsigned long int dpt = (((data[2])<<8)|data[3]);
  for(i=0;udp_table[i].typeid!=0;i++)
  {
    if(spt==udp_table[i].typeid) 
    {
      dissector = i;
      break;
    }
    if(dpt==udp_table[i].typeid) 
    {
      dissector = i;
      break;
    }
  }
  
  //if high verbosity
  if(verbosity ==3)
    vga_output("\n    User Datagram Protocol\n");
   
  if(verbosity == 2)
  {
    vga_output("UDP Port "); 
  } 
    
  //if high or medium verbosity
  if(verbosity ==2 || verbosity ==3)
  {
    if(verbosity ==3)
      vga_output("    Port ");
    vga_dec_output(spt);
    vga_write_output(ARROWRT);
    if(verbosity ==3)
      vga_output("Port ");
    vga_dec_output(dpt);
  }
  
  if(verbosity == 2)
  {
    if(dissector != -1)
    {
      vga_write_output(SPACE);
      vga_output(udp_table[dissector].name);
    }
    else
    {
      vga_output(" (Unknown Protocol)"); 
    }
  }
  
  //if high verbosity
  if (verbosity ==3)
  {
    if(dissector != -1)
    {
      vga_write_output(SPACE);
      vga_output(udp_table[dissector].name);
    }
    vga_output("\n    Checksum: ");
    vga_hex_output(data[6],1);
    vga_hex_output(data[7],0);
    vga_output("  Length: ");
    vga_hex_output(length,1);
  }

  if (dissector != -1)
  {
    if(udp_table[dissector].dissectFcn != 0)
      (udp_table[i].dissectFcn)(data+length,len-length);
  }
}

//parse FTP/////////////////////////////////////////////////////////////////////
void parse_ftp_data(unsigned char* data, unsigned int len)
{
  if(verbosity !=0)
  {
    vga_output("\n      File Transfer Protocol\n");
  }
};

//parse FTP/////////////////////////////////////////////////////////////////////
void parse_ftp_control(unsigned char* data, unsigned int len)
{
  if(verbosity !=0)
  {
    vga_output("\n      File Transfer Protocol\n");
  }
};

//parse Telnet//////////////////////////////////////////////////////////////////
void parse_telnet(unsigned char* data, unsigned int len)
{
  if(verbosity !=0)
  {
    vga_output("\n      Telnet Session\n");
  }
};

//parse SMTP////////////////////////////////////////////////////////////////////
void parse_smtp(unsigned char* data, unsigned int len)
{
  if(verbosity !=0)
  {
    vga_output("\n      Simple Mail Transfer Protocol");
  }
};

//parse DNS/////////////////////////////////////////////////////////////////////
void parse_dns(unsigned char* data, unsigned int len)
{
  if(verbosity !=0)
  {
    vga_output("\n      Domain Name Service\n");
  }
};

//parse Bootstrap protocol/DHCP/////////////////////////////////////////////////
void parse_bootp_dhcp_server(unsigned char* data, unsigned int len)
{
  if(verbosity !=0)
  {
    vga_output("\n      Dynamic Host Configuration Protocol\n");
  }
};

//parse Bootstrap protocol/DHCP/////////////////////////////////////////////////
void parse_bootp_dhcp_client(unsigned char* data, unsigned int len)
{
  if(verbosity !=0)
  {
    vga_output("\n      Dynamic Host Configuration Protocol\n");
  }
};

//parse TFTP////////////////////////////////////////////////////////////////////
void parse_tftp(unsigned char* data, unsigned int len)
{
  if(verbosity !=0)
  {
    vga_output("\n      Trivial File Transfer Protocol\n");
  }
};

//parse HTTP////////////////////////////////////////////////////////////////////
void parse_http(unsigned char* data, unsigned int len)
{
  if(verbosity !=0)
  {
    vga_output("\n      HyperText Transfer Protocol\n");
  }
};

//parse POP3////////////////////////////////////////////////////////////////////
void parse_pop3(unsigned char* data, unsigned int len)
{
  if(verbosity !=0)
  {
    vga_output("\n      Post Office Protocol version 3\n");
  }
};

//parse NTP/////////////////////////////////////////////////////////////////////
void parse_ntp(unsigned char* data, unsigned int len)
{
  if(verbosity !=0)
  {
    vga_output("\n      Network Time Protocol\n");
  }
};

//parse IMAP////////////////////////////////////////////////////////////////////
void parse_imap(unsigned char* data, unsigned int len)
{
  if(verbosity !=0)
  {
    vga_output("\n      Internet Message Access Protocol\n");
  }
};

//parse AIM conversations///////////////////////////////////////////////////////
//TODO- parse actual AIM messages
void parse_aim(unsigned char* data, unsigned int len)
{
  if(verbosity !=0)
  {
    unsigned int i=0;
    vga_output("\n      AOL Instant Messenger\n");
    vga_output("      FNAC family: ");
    
    //use FNAC subtype to determine message type
    long unsigned int type = (((data[6])<<8)|(data[7]));
    long unsigned int subtype = (((data[8])<<8)|(data[9]));
    switch(type)
    {
      case 0x01:
        vga_output("AIM generic\n      FNAC subtype: ");
        vga_hex_output(subtype,1);
        vga_output("Unknown");
        break;
      case 0x02:
        vga_output("AIM location\n      FNAC subtype: ");
        switch(subtype)
        {
          case 0x04:
            vga_output("Set User Info");
            break;
          case 0x06:
            vga_output("User Info Reply");
            break;
          case 0x15:
            vga_output("User Info Query");
            break;
        }
        break;
      case 0x04:
        vga_output("AIM messaging\n      FNAC subtype:");
        switch(subtype)
        {
          case 0x01:
            vga_output("Error");
            break;
          case 0x04:
            vga_output("DL Buddy Icon");
            break;
          case 0x6:
            vga_output("Outgoing");
            vga_output("\n      Buddy Name: ");
            for(i=27;i<(27+data[26]);i++)
            {
              vga_write_output(data[i]); 
            }
            vga_return_output();
            break;
          case 0x7:
            vga_output("Incoming");
            vga_output("\n      Buddy Name: ");
            for(i=27;i<(27+data[26]);i++)
            {
              vga_write_output(data[i]); 
            }
            vga_output("\n      Message: ");
            break;
          case 0x0C:
            vga_output("Acknowledge");
            break;
          case 0x14:
            vga_output("Mini Typing Notifications");
            break;
          default:
            vga_output("Unknown");
            break;
        }
        break;
      default:
        vga_output("Unknown");
    }
  }
};

#endif
