//altera-generated system libraries/////////////////////////////////////////////
#include <stdlib.h>
#include "system.h"
#include "sys/alt_sys_init.h"
#include "sys/alt_irq.h"
#include "sys/alt_alarm.h"
#include "alt_types.h"
#include "priv/alt_file.h"

//static global variable definitions////////////////////////////////////////////
unsigned short int hours = 0, minutes = 0, seconds = 0;
unsigned short int eth_status = 0, pkts_rxd=0, pkts_drpd=0;
unsigned char ether_addr[6]={ 0x00, 0xFA, 0xCA, 0xDE, 0x00, 0x00  };
unsigned short int verbosity = 3, logging = 0, status_bar = 0, capturing=1;
extern int idle_time;

//low-level system hardware interfaces//////////////////////////////////////////
#include "lcd.h"
#include "led.h"
#include "vga.h"
#include "ethernet.h"
#include "keyboard.h"

//functional units//////////////////////////////////////////////////////////////
#include "cmds.h"
#include "proc.h"

//function declarations/////////////////////////////////////////////////////////
int main (void) __attribute__ ((weak, alias ("alt_main")));
void handle_per_sec_ints();

//application entry point - initialize everything///////////////////////////////
int alt_main(void)
{ 
  //initialize the interrupt system
  alt_irq_init(ALT_IRQ_BASE);
  
  // Initialize all the peripherals  
  led_init();
  lcd_init();
  vga_init(); 
  kybd_init();   
  
  //print splash screens & system info
  splash("nohelp");
  sysinfo("nohelp");
  
  //wait a tick
  usleep(5000000);
 
  //Initializing...
  vga_output("Initializing:\n");
 
  //init process monitor
  proc_init();
  vga_write_output(CHECKMARK);
  vga_output("CPU usage monitor initialized.\n");

  //initialize flash interface
  log_init();
  vga_write_output(CHECKMARK);
  vga_output("CFI Flash Initialized.\n");
  
  //bring the ethernet controller online
  DM9000_init();
  alt_irq_register(DM9000A_IRQ, NULL, (void*)handle_eth_ints);           //register ethernet ISR  
  vga_write_output(CHECKMARK);
  vga_output("Ethernet controller initialized.\n");
  
  //register keyboard ISR
  alt_irq_register(KEYBOARD_IRQ, NULL, (void*)kybd_read);               //register keyboard interrupt handler
  IOWR(KEYBOARD_BASE,2,0x100);                                          //set interrupt mask for enter key
  vga_write_output(CHECKMARK);
  vga_output("Keyboard enabled.\n");

  //register ISRs and GO!
  alt_irq_register(PER_SEC_TIMER_IRQ, NULL, (void*)handle_per_sec_ints); //register per-second timer ISR
  IOWR(PER_SEC_TIMER_BASE, 0, 0x01);                                     //clear per-second timer interrupt
  IOWR(PER_SEC_TIMER_BASE, 1, 0x07);                                     //start timer running
  vga_write_output(CHECKMARK);
  vga_output("System run timer activated.\n\n");
  
  //inform the user that the system is ready.
  vga_write_output(ARROWRT);
  vga_output("System ready.\n\n");

  //wait for user input
  while(1)
  {
    asm("nop;");
  };
}

//function handle per_sec ints - updates status bar, etc.///////////////////////
void handle_per_sec_ints()
{ 
  int shift = 0, i=0;
  alt_irq_disable(PROCESS_TIMER_IRQ);

  //update uptime
  seconds++;
  if (seconds == 60)
  {
    seconds = 0;
    minutes++;
  }
  if (minutes == 60)
  {
    minutes = 0;
    hours++; 
  }
  
  //show stats
  if(status_bar==0)
  {
    //update status, packets dropped & uptime
    eth_status = ((ior(NSR)&0xC0)|(ior(NCR)&0x08));
    pkts_drpd+=ior(7);
    if(eth_status & 0x40)
    {
      //update status
      vga_status("\n");
      vga_return_status();
      vga_write_status(GRNSQ);
      vga_status(" Link is up "); 
      if(eth_status&0x80)
      {
        vga_status("10M HDX  ");
      }
      else
      {
        vga_status("100M ");
        if(eth_status&0x08)
        {
          vga_status("FDX ");
        }
        else
        {
          vga_status("HDX ");
        }
      }
    }
    else
    {
      vga_return_status();
      vga_write_status(REDSQ);
      vga_status(" Link Disconnected  ");
    }
    vga_write_status(BLUSQ);
    vga_status(" Received: ");
    vga_dec_status(pkts_rxd,4);
    vga_write_status(SPACE);
    vga_write_status(REDSQ);
    vga_status(" Dropped: ");
    vga_dec_status(pkts_drpd,4);
    vga_write_status(SPACE);
    vga_write_status(GRNSQ);
    vga_status(" Runtime: ");
    vga_dec_status(hours,1);
    vga_write_status(':');
    vga_dec_status(minutes,1);
    vga_write_status(':');
    vga_dec_status(seconds,1);
  }
  //show the cpu meter
  else
  {
    vga_status("\nCPU usage: ");
    shift=(1000-idle_time)*59/1000;
    vga_dec_status((1000-idle_time)/10,2);
    vga_write_status('.');
    vga_dec_status((1000-idle_time)%10,0);
    vga_write_status('%');
    vga_write_status(GRNSQ);
    for(i=0;i<shift;i++)
      vga_write_status(GRNSQ);
  }

  //update process counters, LEDS
  shift = (idle_time)*17/1000;
  led_red((0xFFFFF)<<shift);
  proc_reset();
   
  //clear interrupt flag
  IOWR(PER_SEC_TIMER_BASE, 0, 0x01);
  alt_irq_enable(PROCESS_TIMER_IRQ);

}
