#ifndef FILTER_C
#define FILTER_C

#include "filter.h"

//array of empty filter rules///////////////////////////////////////////////////
struct filterRule filter_Rules[5] = 
{
  {0,"","","","","","","","",0},
  {0,"","","","","","","","",0},
  {0,"","","","","","","","",0},
  {0,"","","","","","","","",0},
  {0,"","","","","","","","",0}
};

//is the filter on?/////////////////////////////////////////////////////////////
int filter_On = FILTER_ACTIVE;

//apply all active rules to the data, return whether packet should be dropped///
int ApplyFilters(unsigned char * data, unsigned int * len)
{
  
  //if filtering is inactive, pass all packets
  if(filter_On == FILTER_INACTIVE)
  { 
    return FILTER_PASS;
  }
  
  unsigned short int i; unsigned int rulesactive = 0;
  
  //rules are ORed together, if it matches any one rule it passes
  for(i=0;i<5;i++)
  {
    switch(ApplyFilterRule(data,len,i))
    {
      case FILTER_PASS:
        return FILTER_PASS; 
        break;
      case FILTER_DROP:
        rulesactive = 1;
        break;
      default:
        break;
    }
  }
  
  //if no rules are active, pass
  if(rulesactive==0)
  {
    return FILTER_PASS;
  }
  
  
  //default action is to drop
  return FILTER_DROP;
}

//apply a single filter rule, used by ApplyFilters, return whether pkt should be dropped
int ApplyFilterRule(unsigned char *data, unsigned int * len, unsigned int ruleNum)
{
  unsigned short int i;
  
  //subrules are ANDed together, it gets dropped unless it matches all subrules
  
  //if rule not active, ignore this rule
  if((filter_Rules[ruleNum].active & FILTER_RULE_ACTIVE) == 0)
  {
    return FILTER_INACTIVE;
  }
    
  //else,check dest mac if active
  if((filter_Rules[ruleNum].active & FILTER_DESTMAC_ACTIVE) == FILTER_DESTMAC_ACTIVE)
  {
    for(i=0;i<6;i++)
    {
      if(data[i] != (filter_Rules[ruleNum].dest_mac[i]))
      {
        return FILTER_DROP;
      }
    }
  }
  
  //check source mac if active
  if((filter_Rules[ruleNum].active & FILTER_SRCMAC_ACTIVE) == FILTER_SRCMAC_ACTIVE)
  {
    for(i=0;i<6;i++)
    {
      if(data[i+6]!=filter_Rules[ruleNum].src_mac[i])
      {
        return FILTER_DROP; 
      }
    }
  }
  
  //if either ip filter active and not an ip packet, drop
  if((filter_Rules[ruleNum].active & (FILTER_SRCIP_ACTIVE|FILTER_DESTIP_ACTIVE)) && ((data[12]!=0x08)|(data[13]!=0x00)))
  {
    return FILTER_DROP;
  }
  
  //check src ip if active - and each with netmask before comparing
  if((filter_Rules[ruleNum].active & FILTER_SRCIP_ACTIVE)==FILTER_SRCIP_ACTIVE)
  {
    //loop over each byte
    for(i=0;i<4;i++)
    {
      if((data[26+i] & filter_Rules[ruleNum].src_netmask[i])!=(filter_Rules[ruleNum].src_netmask[i] & filter_Rules[ruleNum].src_ip[i]))
      {
        return FILTER_DROP;
      }
    } 
  }
  
  //check dest ip if active - and each with netmask before comparing
  if((filter_Rules[ruleNum].active & FILTER_DESTIP_ACTIVE)==FILTER_DESTIP_ACTIVE)
  {
    //loop over each byte
    for(i=0;i<4;i++)
    {
      if((data[30+i] & filter_Rules[ruleNum].dest_netmask[i])!=(filter_Rules[ruleNum].dest_netmask[i] & filter_Rules[ruleNum].dest_ip[i]))
      {
        return FILTER_DROP;
      }
    } 
  }
  
  //get ip header length
  unsigned short int ipheadlen = (data[14] & 0x0F)<<2;
  
  //check src port if active
  if((filter_Rules[ruleNum].active & FILTER_SPORT_ACTIVE)==FILTER_SPORT_ACTIVE)
  {
    if(data[14+ipheadlen]!=filter_Rules[ruleNum].src_port[0] || data[15+ipheadlen]!=filter_Rules[ruleNum].src_port[1])
    {
      return FILTER_DROP; 
    }
  }
  
  //check dest port if active
  if((filter_Rules[ruleNum].active & FILTER_DPORT_ACTIVE)==FILTER_DPORT_ACTIVE)
  {
    if(data[16+ipheadlen]!=filter_Rules[ruleNum].dest_port[0] || data[17+ipheadlen]!=filter_Rules[ruleNum].dest_port[1])
    {
      return FILTER_DROP; 
    }
  }
  
  //check protocol if active
  if((filter_Rules[ruleNum].active & FILTER_PROTO_ACTIVE)==FILTER_PROTO_ACTIVE)
  {
    switch(filter_Rules[ruleNum].protocol)
    {
      case FILTER_PROTO_ANY:
        break;
      case FILTER_PROTO_ARP:
        if((data[12]!=0x08) ||(data[13]!=0x06))
        {
          return FILTER_DROP;
        }
        break;
      case FILTER_PROTO_IP:
        if((data[12]!=0x08) ||(data[13]!=0x00))
        {
          return FILTER_DROP;
        }
        break;
      case FILTER_PROTO_TCP:
        if((data[12]!=0x08)||(data[13]!=0x00)||(data[23]!=6))
        {
          return FILTER_DROP;
        }
        break;
      case FILTER_PROTO_UDP:
        if((data[12]!=0x08)||(data[13]!=0x00)||(data[23]!=17))
        {
          return FILTER_DROP;
        }
        break;
      default:
        break;
    }
  }
  
  //if we got this far, it's good, pass it
  return FILTER_PASS;
}

#endif
